import torch
import torch.nn as nn
import os

from Janex.janexpython import *

import numpy as np
import nltk
import nltk
from nltk.corpus import wordnet
from nltk.tag import pos_tag

from nltk.stem.porter import PorterStemmer
stemmer = PorterStemmer()

import numpy as np
import random
import json

import torch
import torch.nn as nn
from torch.utils.data import Dataset, DataLoader

if torch.cuda.is_available():
    map_location=torch.device('cuda')
else:
    map_location=torch.device('cpu')

def bag_of_words(tokenized_sentence, words):
    # stem each word
    sentence_words = [stem(word) for word in tokenized_sentence]
    # initialize bag with 0 for each word
    bag = np.zeros(len(words), dtype=np.float32)
    for idx, w in enumerate(words):
        if w in sentence_words:
            bag[idx] = 1

    return bag

class NeuralNet(nn.Module):
    def __init__(self, input_size, hidden_size, num_classes):
        super(NeuralNet, self).__init__()
        self.l1 = nn.Linear(input_size, hidden_size)
        self.l2 = nn.Linear(hidden_size, hidden_size)
        self.l3 = nn.Linear(hidden_size, num_classes)
        self.relu = nn.ReLU()

    def forward(self, x):
        out = self.l1(x)
        out = self.relu(out)
        out = self.l2(out)
        out = self.relu(out)
        out = self.l3(out)
        # no activation and no softmax at the end
        return out

class ChatDataset(Dataset):

    def __init__(self, X_train, y_train):
        self.n_samples = len(X_train)
        self.x_data = X_train
        self.y_data = y_train

    # support indexing such that dataset[i] can be used to get i-th sample
    def __getitem__(self, index):
        return self.x_data[index], self.y_data[index]

    # we can call len(dataset) to return the size
    def __len__(self):
        return self.n_samples

class JanexPT:
    def __init__(self, intents_file_path):
        self.FILE = "data.pth"
        self.Classifier = IntentClassifier()
        self.Classifier.set_intentsfp(intents_file_path)
        nltk.download('punkt')
        nltk.download('wordnet')
        nltk.download('averaged_perceptron_tagger')
        self.intents = self.Classifier.load_intents()
        self.device = torch.device('cpu')
        self.accuracy = None

    def set_device(self, device):
        self.device = torch.device(device)

    def pattern_compare(self, input_string):
        try:
            self.data = torch.load(self.FILE, map_location=map_location)
        except:
            self.trainpt()
            self.data = torch.load(self.FILE, map_location=map_location)
        self.input_size = self.data["input_size"]
        self.hidden_size = self.data["hidden_size"]
        self.output_size = self.data["output_size"]
        self.all_words = self.data['all_words']
        self.tags = self.data['tags']
        self.model_state = self.data["model_state"]
        self.model = NeuralNet(self.input_size, self.hidden_size, self.output_size).to(self.device)
        self.intents = self.Classifier.load_intents()
        self.model.load_state_dict(self.model_state)
        self.model.eval()
        sentence = input_string

        sentence = tokenize(sentence)

        X = bag_of_words(sentence, self.all_words)
        X = X.reshape(1, X.shape[0])
        X = torch.from_numpy(X).to(self.device)

        output = self.model(X)
        _, predicted = torch.max(output, dim=1)

        tag = self.tags[predicted.item()]

        probs = torch.softmax(output, dim=1)
        prob = probs[0][predicted.item()]

        self.accuracy = prob.item()

        for intent in self.intents['intents']:
            if tag == intent["tag"]:
                self.intent = intent
                return intent

    def ResponseGenerator(self, response):
        synonyms = []

        response_list = self.IntentMatcher.tokenize(response)

        for token in response_list:
            token = str(token)
            for syn in wordnet.synsets(token):
                for lemma in syn.lemmas():
                    synonyms.append(lemma.name())

        synonyms = list(set(synonyms))

        synonyms = [s for s in synonyms if s != token]

        new_response = " ".join(synonyms)

        return new_response

    def get_wordnet_pos(self, treebank_tag):
        if treebank_tag.startswith('J'):
            return wordnet.ADJ
        elif treebank_tag.startswith('V'):
            return wordnet.VERB
        elif treebank_tag.startswith('N'):
            return wordnet.NOUN
        elif treebank_tag.startswith('R'):
            return wordnet.ADV
        else:
            return wordnet.NOUN  # Default to Noun

    def generate_response_with_synonyms(self, response, strength):
        response_list = self.IntentMatcher.tokenize(response)
        tagged_response = pos_tag(response_list)
        new_response = []
        prestrength = 0

        for word, tag in tagged_response:
            if prestrength > strength:
                break
            og_word = word
            synsets = wordnet.synsets(word, pos=self.get_wordnet_pos(tag))
            if synsets:
                synonyms = synsets[0].lemmas()  # Use the first synonym
                new_word = synonyms[0].name() if synonyms else word
                if og_word.istitle():
                    new_word = new_word.capitalize()
            else:
                new_word = word
                if og_word.istitle():
                    new_word = new_word.capitalize()

            index = response.find(og_word)

            response = response[:index] + new_word + response[index+len(og_word):]

            prestrength = prestrength + 1

        response = self.IntentMatcher.ResponseGenerator(response)

        return response

    def trainpt(self):
        try:
            open("train.py", "r")
            os.system(f"curl -o data.pth https://raw.githubusercontent.com/Cipher58/intents-file/main/data.pth -#")
        except:
            print("Janex-PyTorch: Train program not detected, downloading the program and the pre-trained model from Github.")
            os.system(f"curl -o train.py https://raw.githubusercontent.com/Cipher58/Janex-PyTorch/main/Stock/train.py -#")
            os.system(f"curl -o data.pth https://raw.githubusercontent.com/Cipher58/intents-file/main/data.pth -#")

    def modify_data_path(self, new_path):
        self.FILE = new_path

# Training Program




# Word Manipulation Toolkit

def tokenize(input_string):
    input_string = str(input_string)

    new_string = ""
    deletion = '''!()-[]{};:'"\,<>./?@#$%^&*_~'''

    input_string = input_string.lower()

    for x in input_string:
        if x not in deletion:
            new_string += x

    words = []

    processed_string = new_string
    words = processed_string.split(" ")

    return words

def stem(input_word):
    suffixes = ["ing", "ly", "ed", "es", "'s", "er", "est", "y", "ily", "able", "ful", "ness", "less", "ment", "ive", "ize", "ous"]
    for suffix in suffixes:
        if input_word.endswith(suffix):
            input_word = input_word[:-len(suffix)]
            break
    return input_word


# Vector Toolkit

import numpy as np

def string_vectorize(input_string):
    ascii_values = [ord(char) for char in input_string]  # Get ASCII values for each character
    ascii_vectors = np.array(ascii_values)              # Convert the list to a numpy array
    return ascii_vectors

def reshape_array_dimensions(array, dimensions):
    target_dim = dimensions
    vector1 = np.resize(array, target_dim)
    return vector1

def calculate_cosine_similarity(vector1, vector2):
    dot_product = np.dot(vector1, vector2)
    norm_vector1 = np.linalg.norm(vector1)
    norm_vector2 = np.linalg.norm(vector2)

    if norm_vector1 == 0 or norm_vector2 == 0:
        return 0  # Handle zero division case

    similarity = dot_product / (norm_vector1 * norm_vector2)
    return similarity

# Intent Classifier Toolkit

class IntentClassifier:
    def __init__(self):
        self.intents_file_path = None
        self.thesaurus_file_path = None
        self.vectors_file_path = None

    def set_vectorsfp(self, vectors_file_path):
        self.vectors_file_path = vectors_file_path
        print(f"Janex: Pre-trained vector file path set to {vectors_file_path}")

    def set_intentsfp(self, intents_file_path):
        self.intents_file_path = intents_file_path

    def set_dimensions(self, dimensions):
        self.dimensions = dimensions

    def load_intents(self):
        with open(self.intents_file_path, "r") as json_file:
            data = json.load(json_file)
            return data

    def load_vectors(self):
        with open(self.vectors_file_path, "r") as json_file:
            data = json.load(json_file)
            return data

    def train_vectors(self):
        vectors = {}

        intents = self.load_intents()

        for intent_class in intents["intents"]:
            for pattern in intent_class["patterns"]:
                if pattern is not None:
                    pattern_tokens = tokenize(pattern)
                    for token in pattern_tokens:
                        token_vectors = string_vectorize(token)
                        token_vectors = reshape_array_dimensions(token_vectors, self.dimensions)
                        token_vectors = token_vectors.tolist()
                        vectors[token] = token_vectors
            for response in intent_class["responses"]:
                if response is not None:
                    response_tokens = tokenize(response)
                    for token in response_tokens:
                        token_vectors = string_vectorize(token)
                        token_vectors = reshape_array_dimensions(token_vectors, self.dimensions)
                        vectors[token] = token_vectors.tolist()

        with open(self.vectors_file_path, "w") as vectors_file:
            json.dump(vectors, vectors_file)

    def classify(self, input_string):
        predefined_vectors = self.load_vectors()
        intents = self.load_intents()

        input_tokens = tokenize(input_string)
        vectors_to_classify = []
        highest_similarity = 0

        for token in input_tokens:
            if token in predefined_vectors:
                token_vectors = predefined_vectors[token]
            else:
                token_vectors = string_vectorize(token)

            token_vectors = reshape_array_dimensions(token_vectors, self.dimensions)
            vectors_to_classify.append(token_vectors)

        for intent_class in intents["intents"]:
            intent_vectors_to_classify = []
            for pattern in intent_class["patterns"]:
                pattern_tokens = tokenize(pattern)
                for token in pattern_tokens:
                    if token in predefined_vectors:
                        pattern_token_vectors = predefined_vectors[token]
                        intent_vectors_to_classify.append(pattern_token_vectors)

            vectors_to_classify = reshape_array_dimensions(vectors_to_classify, self.dimensions)
            intent_vectors_to_classify = reshape_array_dimensions(intent_vectors_to_classify, self.dimensions)

            similarity = calculate_cosine_similarity(vectors_to_classify, intent_vectors_to_classify)

            if similarity > highest_similarity:
                highest_similarity = similarity
                most_similar_class = intent_class

        if most_similar_class:
            return most_similar_class


import json
import numpy as np
import spacy
import torch
import torch.nn as nn
import torch.optim as optim
from JanexUltimate.janexpython import *

class SimpleRNN(nn.Module):
    def __init__(self, input_dim, hidden_dim, output_dim):
        super(SimpleRNN, self).__init__()
        self.rnn = nn.RNN(input_dim, hidden_dim, batch_first=True)
        self.fc = nn.Linear(hidden_dim, output_dim)

    def forward(self, x):
        out, _ = self.rnn(x)
        return out

def decompress(janex_model):
    with open(f"{janex_model}", "rb") as bin_file:
        json_data = bin_file.read()
        loaded_data = json.loads(json_data.decode('utf-8'))
        return loaded_data

class NLG:
    def __init__(self, spacy_model, janex_model):
        self.nlp = spacy.load(spacy_model)
        self.trends_dictionary = decompress(janex_model)
        self.max_tokens = 20
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.rnn_model = SimpleRNN(300, 128, len(self.trends_dictionary))

    def set_device(self, device_type):
        self.device = torch.device(device_type)

    def get_word_vector(self, word):
        return self.nlp(word).vector

    def predict_next_word(self, input_word):
        context_window_size = 3
        context_vectors = np.array([self.get_word_vector(word) for word in self.inputs.split()[-context_window_size:]])

        context_vectors = np.resize(context_vectors, 300)
        # Assuming you have already prepared context_vectors as input
        context_tensor = torch.Tensor(context_vectors).unsqueeze(0)  # Add batch dimension

        output = self.rnn_model(context_tensor)
        _, predicted_idx = torch.max(output, 1)
        best_next_word = list(self.trends_dictionary.keys())[predicted_idx.item()]

        if best_next_word and output[0, predicted_idx.item()] > 0.1:
            return best_next_word

    def generate_sentence(self, input):
        sentence = ""

        for _ in range(self.max_tokens):
            self.inputs = input
            input = sentence[:-1]
            next_word = self.predict_next_word(input)
            sentence = f"{sentence} {next_word}"

        return sentence

import json
import spacy

import os

class NLGTraining:
    def __init__(self):
        self.directory = None
        self.nlp = None

    def set_directory(self, directory):
        self.directory = directory

    def set_spacy_model(self, model):
        self.nlp = spacy.load("en_core_web_md")

    def extract_data_from_file(self, file_path):
        with open(file_path, 'r') as file:
            data = file.read()
            return data

    def cycle_through_files(self):

        TotalData = ""

        for root, dirs, files in os.walk(self.directory):
            for file_name in files:
                if file_name.endswith(".txt"):
                    file_path = os.path.join(root, file_name)
                    data = self.extract_data_from_file(file_path)
                    TotalData += data

        return TotalData

    def train_data(self):
        trends_dictionary = {}

        if self.directory is None:
            print("JanexNLG Error! You need to set the directory in which your .txt files are contained.")
            print("")
            print("Use NLGTraining.set_directory('directory')")
            return 404
        if self.nlp is None:
            print("JanexNLG Error! You need to set your desired spacy model. Please select from en_core_web_sm, en_core_web_md or en_core_web_lg!")
            print("")
            print("Use NLGTraining.set_spacy_model('your_desired_model')")
            return 404

        data = self.cycle_through_files()

        tokens = tokenize(data)
        totalsum = len(tokens)

        for i, token in enumerate(tokens):
            progress = i / totalsum * 100
            print(f"Processing token number {i} of {totalsum} ({progress:.2f}% complete)")
            if token not in trends_dictionary:
                # Get the previous and next tokens if they exist
                prev_token = tokens[i - 1] if i > 0 else None
                next_token = tokens[i + 1] if i < len(tokens) - 1 else None

                # Create a list to store the word before and word after
                context_words = []
                if prev_token:
                    context_words.append(prev_token)
                if next_token:
                    context_words.append(next_token)

                # Compute the vector arrays for context words using spaCy
                context_vectors = []
                for word in context_words:
                    word_doc = self.nlp(word)
                    context_vectors.append(word_doc.vector.tolist())

                # Append the context words and their vectors to the dictionary
                trends_dictionary[token] = {
                    "context_words": context_words,
                    "context_vectors": context_vectors
                }
            else:
                prev_token = tokens[i - 1] if i > 0 else None
                next_token = tokens[i + 1] if i < len(tokens) - 1 else None

                # If the token already exists, add the previous and next words
                if prev_token:
                    trends_dictionary[token]["context_words"].append(prev_token)
                if next_token:
                    trends_dictionary[token]["context_words"].append(next_token)

        # Save the trends_dictionary to a JSON file with the desired structure
        output_dictionary = {}
        for token, context_info in trends_dictionary.items():
            output_dictionary[token] = context_info

        with open("custom_janexnlg_model.bin", "wb") as bin_file:
            json_data = json.dumps(output_dictionary).encode('utf-8')
            bin_file.write(json_data)

    def finetune_model(self, model_name):
        with open(model_name, "rb") as bin_file:
            json_data = bin_file.read()
            trends_dictionary = json.loads(json_data.decode('utf-8'))

        if self.directory is None:
            print("JanexNLG Error! You need to set the directory in which your .txt files are contained.")
            print("")
            print("Use NLGTraining.set_directory('directory')")
            return 404
        if self.nlp is None:
            print("JanexNLG Error! You need to set your desired spacy model. Please select from en_core_web_sm, en_core_web_md or en_core_web_lg!")
            print("")
            print("Use NLGTraining.set_spacy_model('your_desired_model')")
            return 404

        data = self.cycle_through_files()

        tokens = tokenize(data)
        totalsum = len(tokens)

        for i, token in enumerate(tokens):
            progress = i / totalsum * 100
            print(f"Processing token number {i} of {totalsum} ({progress:.2f}% complete)")
            if token not in trends_dictionary:
                # Get the previous and next tokens if they exist
                prev_token = tokens[i - 1] if i > 0 else None
                next_token = tokens[i + 1] if i < len(tokens) - 1 else None

                # Create a list to store the word before and word after
                context_words = []
                if prev_token:
                    context_words.append(prev_token)
                if next_token:
                    context_words.append(next_token)

                # Compute the vector arrays for context words using spaCy
                context_vectors = []
                for word in context_words:
                    word_doc = self.nlp(word)
                    context_vectors.append(word_doc.vector.tolist())

                # Append the context words and their vectors to the dictionary
                trends_dictionary[token] = {
                    "context_words": context_words,
                    "context_vectors": context_vectors
                }
            else:
                prev_token = tokens[i - 1] if i > 0 else None
                next_token = tokens[i + 1] if i < len(tokens) - 1 else None

                # If the token already exists, add the previous and next words
                if prev_token:
                    trends_dictionary[token]["context_words"].append(prev_token)
                if next_token:
                    trends_dictionary[token]["context_words"].append(next_token)

        # Save the trends_dictionary to a JSON file with the desired structure
        output_dictionary = {}
        for token, context_info in trends_dictionary.items():
            output_dictionary[token] = context_info

        with open("custom_janexnlg_model.bin", "wb") as bin_file:
            json_data = json.dumps(output_dictionary).encode('utf-8')
            bin_file.write(json_data)


import json
import random
import os
import string
from Cipher import *

import random
import spacy
from spacy.lang.en import English

from Janex.janexpython import *
import numpy as np

class JanexSpacy:
    def __init__(self, intents_file_path, thesaurus_file_path, vectors_file_path):
        self.intents_file_path = intents_file_path
        self.vectors_file_path = vectors_file_path
        self.JVanilla = IntentMatcher(intents_file_path, thesaurus_file_path)
        self.nlp = spacy.load("en_core_web_sm")
        self.intents = self.JVanilla.train()
        self.intentmatcher = IntentMatcher(intents_file_path, thesaurus_file_path)

        # Load or compute and save vectors
        if os.path.exists(vectors_file_path):
            with open(vectors_file_path, "r") as vectors_file:
                self.pattern_vectors = json.load(vectors_file)
        else:
            print(f"JanexSC: Your intents haven't been compiled into vectors. Automatically training your current {self.intents_file_path} data into {self.vectors_file_path}.")
            self.trainvectors()

        self.misunderstanding = ["Sorry, I did not understand what you asked.", "Sorry, I'm not programmed to talk about that.", "I couldn't find a match to what you said in my database.", "Sorry, I didn't quite catch that.", "Sorry, I did not interpret what you said."]

        self.vector_dim = 300

        self.pattern_vectors = None

    def load_vectors(self):
        with open(self.vectors_file_path, "r") as vector_file:
            pattern_vectors = json.load(vector_file)
            return pattern_vectors

    def pattern_compare(self, input_string):
        if self.pattern_vectors is None:
            self.pattern_vectors = self.load_vectors()
        highest_similarity = 0
        most_similar_pattern = None
        threshold = 0.085

        input_tokens = self.intentmatcher.tokenize(input_string)
        input_vector = np.zeros(self.vector_dim)

        for token in input_tokens:
            if token in self.pattern_vectors:
                token_vector = self.pattern_vectors[token]
                if len(token_vector) != self.vector_dim:
                    # Resize the token_vector to match the dimension of input_vector
                    token_vector = np.resize(token_vector, self.vector_dim)
                input_vector += token_vector
            else:
                token_vector = self.nlp(token).vector
                if len(token_vector) != self.vector_dim:
                    # Resize the token_vector to match the dimension of input_vector
                    token_vector = np.resize(token_vector, self.vector_dim)
                input_vector += token_vector

        for intent_class in self.intents["intents"]:
            patterns = intent_class["patterns"] if intent_class else []

            for pattern in patterns:
                if pattern:
                    if pattern in self.pattern_vectors:
                        pattern_vector = np.array(self.pattern_vectors[pattern])
                        similarity = self.calculate_cosine_similarity(input_vector, pattern_vector)

                        if similarity > highest_similarity:
                            highest_similarity = similarity
                            most_similar_pattern = intent_class

        if most_similar_pattern:
            return most_similar_pattern
        else:
            msp, sim = self.intentmatcher.pattern_compare(input_string)
            return msp

    def response_compare(self, input_string, intent_class):
        highest_similarity = 0
        most_similar_response = None
        threshold = 0.085

        responses = intent_class["responses"] if intent_class else []

        input_tokens = self.intentmatcher.tokenize(input_string)

        input_vector = np.zeros(self.vector_dim)  # Initialize an empty vector

        for token in input_tokens:
            if token in self.pattern_vectors:
                token_vector = self.pattern_vectors[token]
                if len(token_vector) != self.vector_dim:
                    # Resize the token_vector to match the dimension of input_vector
                    token_vector = np.resize(token_vector, self.vector_dim)
                input_vector += token_vector
            else:
                pass

        for response in responses:
            if response:
                if response in self.pattern_vectors:
                    response_vector = np.array(self.pattern_vectors[response])
                    similarity = self.calculate_cosine_similarity(input_vector, response_vector)

                    if similarity > highest_similarity:
                        highest_similarity = similarity
                        most_similar_response = response

        if most_similar_response is not None:
            return most_similar_response
        else:
            most_similar_response = self.intentmatcher.response_compare(input_string, intent_class)
            return most_similar_response

    def legacy_pattern_compare(self, input_string):
        highest_similarity = 0
        most_similar_pattern = None
        threshold = 0.085

        for intent_class in self.intents["intents"]:

            patterns = intent_class["patterns"] if intent_class else []

            input_strings = self.intentmatcher.tokenize(input_string)
            input_string = " ".join(input_strings)

            input_doc = self.nlp(input_string)

            for pattern in patterns:
                if pattern is not None:
                    pattern_vector = self.nlp(pattern).vector  # Precompute vector
                    try:
                        similarity = self.calculate_cosine_similarity(input_doc.vector, pattern_vector)
                    except:
                        return random.choice(self.misunderstanding)
                if similarity > highest_similarity:
                    highest_similarity = similarity
                    most_similar_pattern = intent_class
                else:
                    pass

        if most_similar_pattern and highest_similarity > threshold:
            return most_similar_pattern
        else:
            return random.choice(self.misunderstanding)

    def legacy_response_compare(self, input_string, intent_class):
        highest_similarity = 0
        most_similar_response = None
        threshold = 0.085

        responses = intent_class["responses"] if intent_class else []

        input_strings = self.intentmatcher.tokenize(input_string)
        input_string = " ".join(input_strings)

        input_doc = self.nlp(input_string)

        for response in responses:
            if response is not None:
                response_vector = self.nlp(response).vector  # Precompute vector
                try:
                    similarity = self.calculate_cosine_similarity(input_doc.vector, response_vector)
                except:
                    return random.choice(self.misunderstanding)

                if similarity > highest_similarity:
                    highest_similarity = similarity
                    most_similar_response = response
            else:
                pass

        if most_similar_response and highest_similarity > threshold:
            return most_similar_response
        else:
            most_similar_response = self.intentmatcher.response_compare(input_string, intent_class)
            return most_similar_response

    def calculate_cosine_similarity(self, vector1, vector2):
        # Resize vectors to a common dimension
        target_dim = 300
        vector1 = np.resize(vector1, target_dim)
        vector2 = np.resize(vector2, target_dim)

        # Calculate cosine similarity between two vectors
        dot_product = np.dot(vector1, vector2)
        norm_vector1 = np.linalg.norm(vector1)
        norm_vector2 = np.linalg.norm(vector2)

        if norm_vector1 == 0 or norm_vector2 == 0:
            return 0  # Handle zero division case

        similarity = dot_product / (norm_vector1 * norm_vector2)
        return similarity

    def update_thesaurus(self):
        self.intentmatcher.update_thesaurus(self.thesaurus_file_path)

    def Tokenizer(self, input_string):
        words = []
        words = self.intentmatcher.tokenize(input_string)
        return words

    def ResponseGenerator(self, input_string):
        NewResponse = self.intentmatcher.ResponseGenerator(input_string)
        return NewResponse

    def trainvectors(self):
        self.pattern_vectors = {}
        patterncount = 0
        responsecount = 0
        patterntoks = 0
        responsetoks = 0
        for intent_class in self.intents["intents"]:
            for pattern in intent_class["patterns"]:
                patterns = self.intentmatcher.tokenize(pattern)
                for token in patterns:
                    token_vector = self.nlp(token).vector
                    self.pattern_vectors[token] = token_vector.tolist()
                    patterntoks += 1
                patterncount += 1
                print(f"{token}: {token_vector}")

        with open(self.vectors_file_path, "w") as vectors_file:
            json.dump(self.pattern_vectors, vectors_file)

        print(f"JanexSC: Training completed. {patterncount} patterns ({patterntoks} tokens) transformed into vectors.")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="JanexUltimate",
    version="0.1.4",
    author="ChronoByte404",
    author_email="cipher58public@gmail.com",
    description="A description of your package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ChronoByte404/Janex-Ultimate",
    packages=setuptools.find_packages(),
    install_requires=[
        "CipherProgram",  # Specify the minimum version if known
        "torch",
        "nltk",
        "spacy",
    ],
    license="Lily 3.0",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)
